@@ -76,33 +76,28 @@
 
 ;;; Create a stream that works early.
 (defun !make-cold-stderr-stream ()
-  (let ((stderr
-          #-win32 2
-          #+win32 (sb-win32::get-std-handle-or-null sb-win32::+std-error-handle+))
-        (buf (make-string 1 :element-type 'base-char :initial-element #\Space)))
-    (%make-fd-stream
-     :cout (lambda (stream ch)
+  (let* ((stderr
+           #-win32 2
+           #+win32 (sb-win32::get-std-handle-or-null sb-win32::+std-error-handle+))
+         (ucs2 #+win32 (sb-win32::console-handle-p stderr))
+         (char-size (if ucs2 4 1))
+         (buf (make-string 1 :element-type (if ucs2 'character 'base-char) :initial-element #\Space)))
+    (flet ((cout (stream ch)
              (declare (ignore stream))
              (setf (char buf 0) ch)
-             (sb-unix:unix-write stderr buf 0 1)
-             ch)
-     :sout (lambda (stream string start end)
-             (declare (ignore stream))
-             (flet ((out (s start len)
-                      (when (plusp len)
-                        (setf (char buf 0) (char s (+ start len -1))))
-                      (sb-unix:unix-write stderr s start len)))
-               (if (typep string 'simple-base-string)
-                   (out string start (- end start))
-                   (let ((n (- end start)))
-                     ;; will croak if there is any non-BASE-CHAR in the string
-                     (out (replace (make-array n :element-type 'base-char)
-                                   string :start2 start) 0 n)))))
-     :misc (lambda (stream operation arg1)
-             (declare (ignore stream arg1))
-             (stream-misc-case (operation :default nil)
-               (:charpos ; impart just enough smarts to make FRESH-LINE dtrt
-                (if (eql (char buf 0) #\newline) 0 1)))))))
+             (sb-unix:unix-write stderr buf 0 char-size)
+             ch))
+      (%make-fd-stream
+       :cout #'cout
+       :sout (lambda (stream string start end)
+               (declare (simple-string string) (index start end))
+               (loop for i from start below end
+                     do (cout stream (char string i))))
+       :misc (lambda (stream operation arg1)
+               (declare (ignore stream arg1))
+               (stream-misc-case (operation :default nil)
+                                 (:charpos ; impart just enough smarts to make FRESH-LINE dtrt
+                                  (if (eql (char buf 0) #\newline) 0 1))))))))
 
 (defun !xc-sanity-checks ()
   ;; Verify on startup that some constants were dumped reflecting the
@@ -229,6 +224,8 @@
 
   (setq sb-pcl::*!docstrings* nil) ; needed before any documentation is set
   (setq sb-c::*queued-proclaims* nil) ; needed before any proclaims are run
+  (setq *code-coverage-info* ; needed to note / record code coverage
+        (list (make-hash-table :test 'equal :synchronized t)))
 
   (/show0 "calling cold toplevel forms and fixups")
   (let ((*package* *package*)) ; rebind to self, as if by LOAD
@@ -257,6 +254,13 @@
            (sb-fasl::named-constant-set object index name)))
         ((cons (eql :begin-file))
          (unless (!c-runtime-noinform-p) (print (cdr toplevel-thing))))
+        ((cons (eql :note-code-covered))
+         (aver (typep (cdr toplevel-thing) 'code-component))
+         (push (make-weak-pointer (cdr toplevel-thing))
+               (cdr *code-coverage-info*)))
+        ((cons (eql :record-code-coverage))
+         (setf (gethash (second toplevel-thing) (car *code-coverage-info*))
+               (mapcar #'list (third toplevel-thing))))
         (t
          (!cold-lose "bogus operation in *!COLD-TOPLEVELS*")))))
   (/show0 "done with loop over cold toplevel forms and fixups")
@@ -264,11 +268,6 @@
 
   (makunbound '*!cold-toplevels*) ; so it gets GC'd
 
-  ;; Need the static-space replica of the assembly routine jump vector
-  ;; filled in. This matters only for code that gets compiled to dynamic space,
-  ;; so it's OK that it occurs somewhat late in cold-init.
-  #+x86-64 (sb-vm::validate-asm-routine-vector)
-
   #+win32 (show-and-call reinit-internal-real-time)
 
   ;; Set sane values again, so that the user sees sane values instead
@@ -330,6 +329,7 @@
   (show-and-call sb-disassem::!compile-inst-printers)
 
   ;; Toggle some readonly bits
+  #-sb-devel
   (dovector (sc sb-c:*backend-sc-numbers*)
     (when sc
       (logically-readonlyize (sb-c::sc-move-funs sc))
@@ -415,7 +415,6 @@ process to continue normally."
     ;; can be called, as pretty much anything can assume that it is set.
     (when total ; newly started process, and not a failed save attempt
       (sb-thread::init-main-thread)
-      #+x86-64 (sb-vm::validate-asm-routine-vector)
       (rebuild-package-vector))
     ;; Initialize streams next, so that any errors can be printed
     (stream-reinit t)
