@@ -270,8 +270,8 @@
               (values not-target target)
               (values target not-target))
         (assemble ()
-          (inst asr r value n-fixnum-tag-bits)
           (when fixnum-p
+            (inst asr r value n-fixnum-tag-bits)
             (inst tbz* value 0 yep))
           (unless (fixnum-or-other-pointer-tn-ref-p args t)
             (test-type value temp nope t (other-pointer-lowtag)))
@@ -357,63 +357,70 @@
     NOT-TARGET))
 
 (define-vop (unsigned-byte-x-p type-predicate)
-  (:arg-types * (:constant (integer 1)))
+  (:arg-types * (:constant t))
   (:translate sb-c::unsigned-byte-x-p)
   (:info target not-p x)
   (:temporary (:sc unsigned-reg) last-digit)
   (:generator 10
-    (let* ((type (tn-ref-type args))
-           (fixnum-p (types-equal-or-intersect type (specifier-type 'fixnum)))
-           (integer-p (csubtypep type (specifier-type 'integer)))
-           (other-pointer-p (fixnum-or-other-pointer-tn-ref-p args t))
-           (unsigned-p (not (types-equal-or-intersect type (specifier-type '(integer * -1))))))
-      (multiple-value-bind (yep nope)
-          (if not-p
-              (values not-target target)
-              (values target not-target))
-        (assemble ()
-          (cond ((not other-pointer-p)
-                 ;; Move to a temporary and mask off the lowtag,
-                 ;; but leave the sign bit for testing for positive fixnums.
-                 ;; When using 32-bit registers that bit will not be visible.
-                 (inst and last-digit value (logior (ash 1 (1- n-word-bits)) lowtag-mask)))
-                (fixnum-p
-                 (move last-digit value)))
-          (when fixnum-p
-            (%test-fixnum last-digit nil (if unsigned-p
-                                             yep
-                                             fixnum) nil))
-          (unless other-pointer-p
-            (inst cmp (32-bit-reg last-digit) other-pointer-lowtag)
-            (inst b :ne nope))
-          ;; Get the header.
-          (loadw temp value 0 other-pointer-lowtag)
-          (unless integer-p
-            (inst and tmp-tn temp widetag-mask)
-            (inst cmp tmp-tn bignum-widetag)
-            (inst b :ne nope))
-          #.(assert (= (integer-length bignum-widetag) 5))
-          (inst add last-digit value (lsr temp 5))
-          (inst ldr last-digit (@ last-digit (- other-pointer-lowtag)))
-          (inst lsr temp temp n-widetag-bits)
-          (inst cmp temp (add-sub-immediate (1+ (/ x n-word-bits))))
-          (inst b :gt nope)
-          (inst b :lt (if unsigned-p
-                          yep
-                          fixnum))
-          ;; Is it a sign-extended sign bit
-          (cond ((not unsigned-p)
-                 (inst cbnz last-digit nope))
-                (not-p
-                 (inst cbnz last-digit target))
-                (t
-                 (inst cbz last-digit target)))
-
-          fixnum
-          (unless unsigned-p
+    (multiple-value-bind (digits left) (truncate x n-word-bits)
+      (let* ((type (tn-ref-type args))
+             (fixnum-p (types-equal-or-intersect type (specifier-type 'fixnum)))
+             (integer-p (csubtypep type (specifier-type 'integer)))
+             (other-pointer-p (fixnum-or-other-pointer-tn-ref-p args t))
+             (unsigned-p (not (types-equal-or-intersect type (specifier-type '(integer * -1))))))
+        (multiple-value-bind (yep nope)
             (if not-p
-                (inst tbnz* last-digit (1- n-word-bits) target)
-                (inst tbz* last-digit (1- n-word-bits) target))))))
+                (values not-target target)
+                (values target not-target))
+          (assemble ()
+            (cond ((not other-pointer-p)
+                   ;; Move to a temporary and mask off the lowtag,
+                   ;; but leave the sign bit for testing for positive fixnums.
+                   ;; When using 32-bit registers that bit will not be visible.
+                   (inst and last-digit value (logior (ash 1 (1- n-word-bits)) lowtag-mask)))
+                  (fixnum-p
+                   (move last-digit value)))
+            (when fixnum-p
+              (%test-fixnum last-digit nil (if unsigned-p
+                                               yep
+                                               fixnum) nil))
+            (unless other-pointer-p
+              (inst cmp (32-bit-reg last-digit) other-pointer-lowtag)
+              (inst b :ne nope))
+            ;; Get the header.
+            (loadw temp value 0 other-pointer-lowtag)
+            (unless integer-p
+              (inst and tmp-tn temp widetag-mask)
+              (inst cmp tmp-tn bignum-widetag)
+              (inst b :ne nope))
+            #.(assert (= (integer-length bignum-widetag) 5))
+            (inst add last-digit value (lsr temp 5))
+            (inst ldr last-digit (@ last-digit (- other-pointer-lowtag)))
+            (inst lsr temp temp n-widetag-bits)
+            (inst cmp temp (add-sub-immediate (1+ digits)))
+            (inst b :gt nope)
+            (inst b :lt (if unsigned-p
+                            yep
+                            fixnum))
+            (if (zerop left)
+                ;; Is it a sign-extended sign bit
+                (cond ((not unsigned-p)
+                       (inst cbnz last-digit nope))
+                      (not-p
+                       (inst cbnz last-digit target))
+                      (t
+                       (inst cbz last-digit target)))
+                (progn
+                  ;; Check if the remaining high bits are zero
+                  (inst tst last-digit (dpb 0 (byte left 0) -1))
+                  (inst b :eq yep)
+                  (inst b nope)))
+
+            fixnum
+            (unless unsigned-p
+              (if not-p
+                  (inst tbnz* last-digit (1- n-word-bits) target)
+                  (inst tbz* last-digit (1- n-word-bits) target)))))))
     NOT-TARGET))
 
 (define-vop (unsigned-byte-64-p-move-to-word unsigned-byte-64-p)
@@ -525,6 +532,15 @@
         (inst b unsigned-fall-through)))
     not-target))
 
+(define-vop (fixnump)
+  (:translate fixnump)
+  (:arg-refs arg-ref)
+  (:args (value :scs (any-reg descriptor-reg)))
+  (:conditional :eq)
+  (:policy :fast-safe)
+  (:generator 4
+    (inst tst value n-fixnum-tag-bits)))
+
 (define-vop (fixnump/unsigned)
   (:policy :fast-safe)
   (:args (value :scs (unsigned-reg)))
@@ -767,12 +783,12 @@
 
 (define-vop (test-widetag)
   (:args (value :scs (unsigned-reg)))
-  (:info target not-p type-codes)
+  (:info target not-p type-codes object-tn-ref)
   (:generator 1
     (%test-headers value nil target not-p nil
       (if (every #'integerp type-codes)
           (canonicalize-widetags type-codes)
-          type-codes))))
+          type-codes) :value-tn-ref object-tn-ref)))
 
 (define-vop (load-instance-layout)
   (:args (object :scs (any-reg descriptor-reg)))
