@@ -458,7 +458,9 @@
                 ;; We can't stuff the BINDS as &AUX vars into the lambda list
                 ;; because there can be a RETURN-FROM in there.
                 (defun ,name (,node ,@vars &rest ,args)
-                  (declare (ignorable ,node ,@(butlast vars))
+                  (declare (ignorable ,node ,@(remove-if (lambda (v)
+                                                           (member v lambda-list-keywords))
+                                                         (butlast vars)))
                            (ignore ,args))
                   ,@(if var-decls (list var-decls))
                   (binding* (,@binds)
@@ -1029,7 +1031,10 @@ specify bindings for printer control variables.")
                                      (if (and (eq car (car subtree))
                                               (eq cdr (cdr subtree)))
                                          subtree
-                                         (cons car cdr))))))))
+                                         (if (typep car '(cons (eql $when)))
+                                             (and (eval (second car))
+                                                  (cddr car))
+                                             (cons car cdr)))))))))
                (s tree)))
            (test (pattern with)
              (lambda (x)
