@@ -407,3 +407,25 @@
   (let ((string (with-output-to-string (s)
                   (example-method-combination-order-nil-gf t s))))
     (assert (string= string "1: T and 2: T"))))
+
+(defmethod one-required-argument ((x standard-object))
+  (list x))
+(defmethod one-required-argument ((x t))
+  (list t))
+(with-test (:name :one-required-argument-standard-object-not-just-instance)
+  (assert (equal (one-required-argument #'one-required-argument)
+                 (list #'one-required-argument)))
+  (assert (equal (one-required-argument #p"") (list t)))
+  (assert (equal (one-required-argument (make-condition 'error)) (list t))))
+
+(defmethod many-required-arguments ((x standard-object) a b c d e f g h i &rest args)
+  (cons x args))
+(defmethod many-required-arguments ((x t) a b c d e f g h i &rest args)
+  args)
+(with-test (:name :many-required-arguments-standard-object-not-just-instance)
+  (assert (equal (many-required-arguments #'many-required-arguments 1 2 3 4 5 6 7 8 9 10 11 12)
+                 (list #'many-required-arguments 10 11 12)))
+  (assert (equal (many-required-arguments #p"" 1 2 3 4 5 6 7 8 9 10 11 12)
+                 (list 10 11 12)))
+  (assert (equal (many-required-arguments (make-condition 'error) 1 2 3 4 5 6 7 8 9 10 11 12)
+                 (list 10 11 12))))
